/**
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com
 *
 * @author Daniel Rentz <daniel.rentz@open-xchange.com>
 * @author Malte Timmermann <malte.timmermann@open-xchange.com>
 * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
 */

define('io.ox/office/text/main',
    ['io.ox/office/tk/utils',
     'io.ox/office/framework/app/editapplication',
     'io.ox/office/text/editor',
     'io.ox/office/text/view/view',
     'io.ox/office/text/app/controller'
    ], function (Utils, EditApplication, TextModel, TextView, TextController) {

    'use strict';

    // class TextApplication ==================================================

    /**
     * The OX Text application.
     *
     * @constructor
     *
     * @extends EditApplication
     *
     * @param {Object} [appOptions]
     *  A map of static application options, that have been passed to the
     *  static method BaseApplication.createLauncher().
     *
     * @param {Object} [launchOptions]
     *  A map of options to control the properties of the application. Supports
     *  all options supported by the base class EditApplication.
     */
    var TextApplication = EditApplication.extend({ constructor: function (appOptions, launchOptions) {

        var // self reference
            self = this;

        // base constructor ---------------------------------------------------

        EditApplication.call(this, TextModel, TextView, TextController, appOptions, launchOptions, {
            postProcessHandler: processDocument,
            optimizeOperationsHandler: optimizeOperations,
            actionsDelay: 500,
            actionsMaxDelay: 1000
        });

        // private methods ----------------------------------------------------

        /**
         * Post-processes the document contents and formatting, after all its
         * import operations have been applied.
         *
         * @returns {jQuery.Promise}
         *  The promise of a Deferred object that will be resolved when the
         *  document has been post-processed successfully, or rejected when an
         *  error has occurred.
         */
        function processDocument() {
            return self.getModel().updateDocumentFormatting();
        }

        /**
         * Optimizing the actions (operations) before sending them to the server.
         *
         * @param {Object[]} actionsBuffer
         *  An array with actions.
         *
         * @returns {Object[]}
         *  The optimized array with actions.
         */
        function optimizeOperations(actionsBuffer) {
            return self.getModel().optimizeActions(actionsBuffer);
        }

    }}); // class TextApplication

    // exports ================================================================

    return EditApplication.createLauncher('io.ox/office/text', TextApplication, { icon: 'icon-align-left', search: true });

});
